# Generated by roxygen2: do not edit by hand

S3method(autoplot,coco)
S3method(autoplot,cocoBoot)
S3method(autoplot,cocoForecast)
S3method(autoplot,cocoForecastCollection)
S3method(autoplot,cocoPit)
S3method(autoplot,cocoResid)
S3method(coef,coco)
S3method(extractAIC,coco)
S3method(fitted,coco)
S3method(fitted,cocoResid)
S3method(logLik,coco)
S3method(nobs,coco)
S3method(plot,coco)
S3method(plot,cocoBoot)
S3method(plot,cocoForecast)
S3method(plot,cocoForecastCollection)
S3method(plot,cocoPit)
S3method(plot,cocoResid)
S3method(predict,coco)
S3method(print,coco)
S3method(print,cocoBoot)
S3method(print,cocoForecastCollection)
S3method(print,cocoPit)
S3method(print,cocoResid)
S3method(print,summary.coco)
S3method(print,summary.cocoBoot)
S3method(print,summary.cocoForecast)
S3method(print,summary.cocoForecastCollection)
S3method(print,summary.cocoPit)
S3method(print,summary.cocoResid)
S3method(print,summary.cocoSoc)
S3method(residuals,coco)
S3method(residuals,cocoResid)
S3method(rstandard,coco)
S3method(summary,coco)
S3method(summary,cocoBoot)
S3method(summary,cocoForecast)
S3method(summary,cocoForecastCollection)
S3method(summary,cocoPit)
S3method(summary,cocoResid)
S3method(summary,cocoSoc)
S3method(vcov,coco)
export(autoplot)
export(cocoBoot)
export(cocoPit)
export(cocoReg)
export(cocoResid)
export(cocoScore)
export(cocoSim)
export(cocoSoc)
export(installJuliaPackages)
export(setJuliaSeed)
importFrom(HMMpa,dgenpois)
importFrom(JuliaConnectoR,juliaCall)
importFrom(JuliaConnectoR,juliaEval)
importFrom(JuliaConnectoR,juliaGet)
importFrom(JuliaConnectoR,juliaLet)
importFrom(Rcpp,sourceCpp)
importFrom(forecast,Acf)
importFrom(ggplot2,autoplot)
importFrom(ggplot2,ggtitle)
importFrom(ggplot2,theme_bw)
importFrom(matrixStats,rowQuantiles)
importFrom(numDeriv,grad)
importFrom(numDeriv,hessian)
importFrom(stats,coef)
importFrom(stats,constrOptim)
importFrom(stats,extractAIC)
importFrom(stats,fitted)
importFrom(stats,glm)
importFrom(stats,lm)
importFrom(stats,logLik)
importFrom(stats,na.omit)
importFrom(stats,nobs)
importFrom(stats,optim)
importFrom(stats,pchisq)
importFrom(stats,predict)
importFrom(stats,qchisq)
importFrom(stats,residuals)
importFrom(stats,rpois)
importFrom(stats,rstandard)
importFrom(stats,runif)
importFrom(stats,var)
importFrom(stats,vcov)
importFrom(utils,head)
useDynLib(coconots, .registration = TRUE)
